@@ -1,7 +1,7 @@
 #! /usr/bin/env python3
 
 from .base_metric_loss_function import BaseMetricLossFunction
-from ..utils import loss_and_miner_utils as lmu
+from ..utils import loss_and_miner_utils as lmu, common_functions as c_f
 import torch
 
 class NCALoss(BaseMetricLossFunction):
@@ -11,6 +11,8 @@ class NCALoss(BaseMetricLossFunction):
 
     # https://www.cs.toronto.edu/~hinton/absps/nca.pdf
     def compute_loss(self, embeddings, labels, indices_tuple):
+        if len(embeddings) <= 1:
+            return self.zero_losses()
         return self.nca_computation(embeddings, embeddings, labels, labels, indices_tuple)
 
     def nca_computation(self, query, reference, query_labels, reference_labels, indices_tuple):
@@ -22,6 +24,6 @@ class NCALoss(BaseMetricLossFunction):
         same_labels = (query_labels.unsqueeze(1) == reference_labels.unsqueeze(0)).float()
         exp = torch.nn.functional.softmax(self.softmax_scale*x, dim=1)
         exp = torch.sum(exp * same_labels, dim=1)
-        exp = exp * miner_weights
-        non_zero_prob = torch.masked_select(exp, exp != 0)
-        return -torch.mean(torch.log(non_zero_prob))
\ No newline at end of file
+        non_zero = exp!=0
+        loss = -torch.log(exp[non_zero])*miner_weights[non_zero]
+        return {"loss": {"losses": loss, "indices": c_f.torch_arange_from_size(query)[non_zero], "reduction_type": "element"}}
\ No newline at end of file
